@@ -849,7 +849,7 @@ service_introspect (GkdSecretService *self,
 
 	g_ptr_array_add (names, NULL);
 
-	reply = gkd_dbus_introspect_handle (message, gkd_secret_introspect_collection,
+	reply = gkd_dbus_introspect_handle (message, gkd_secret_introspect_service,
 	                                    (const gchar **)names->pdata);
 
 	g_ptr_array_unref (names);
@@ -890,8 +890,9 @@ service_message_handler (GkdSecretService *self, DBusMessage *message)
 	if (dbus_message_is_method_call (message, SECRET_SERVICE_INTERFACE, "Lock"))
 		return service_method_lock (self, message);
 
-	/* org.gnome.keyring.Service.ChangeLock() */
-	if (dbus_message_is_method_call (message, SECRET_SERVICE_INTERFACE, "ChangeLock"))
+	/* org.gnome.keyring.InternalUnsupportedGuiltRiddenInterface.ChangeWithPrompt() */
+	if (dbus_message_is_method_call (message, INTERNAL_SERVICE_INTERFACE, "ChangeWithPrompt") ||
+	    dbus_message_is_method_call (message, SECRET_SERVICE_INTERFACE, "ChangeLock"))
 		return service_method_change_lock (self, message);
 
 	/* org.freedesktop.Secret.Service.ReadAlias() */
