@@ -94,10 +94,10 @@ static void
 setup_module (Test *test,
               gconstpointer unused)
 {
-	CK_ATTRIBUTE label = { CKA_LABEL, NULL, 0 };
 	CK_ATTRIBUTE url = { CKA_URL, NULL, 0 };
 	gchar *contents;
 	gsize length;
+	EggBytes *bytes;
 	GkmManager *manager;
 	GError *error = NULL;
 	GkmSession *session;
@@ -115,8 +115,8 @@ setup_module (Test *test,
 	rv = gkm_gnome2_storage_refresh (test->storage);
 	gkm_assert_cmprv (rv, ==, CKR_OK);
 
+	/* We already have the CKA_LABEL attribute */
 	gkm_store_register_schema (GKM_STORE (test->storage), &url, NULL, 0);
-	gkm_store_register_schema (GKM_STORE (test->storage), &label, NULL, 0);
 
 	/*
 	 * Create a new object that hasn't yet been stored in the storage.
@@ -129,9 +129,12 @@ setup_module (Test *test,
 	                                 NULL);
 	g_file_get_contents (SRCDIR "/files/test-certificate.cer", &contents, &length, &error);
 	g_assert_no_error (error);
-	if (!gkm_serializable_load (GKM_SERIALIZABLE (test->new_object), NULL, contents, length))
+
+	bytes = egg_bytes_new_take (contents, length);
+	if (!gkm_serializable_load (GKM_SERIALIZABLE (test->new_object), NULL, bytes))
 		g_assert_not_reached ();
-	g_free (contents);
+	egg_bytes_unref (bytes);
+
 	/* We happen to know this certificate will get named */
 	test->new_filename = g_build_filename (test->directory, "CA_Cert_Signing_Authority.cer", NULL);
 
@@ -172,6 +175,7 @@ teardown_directory (Test *test,
 		filename = g_build_filename (test->directory, name, NULL);
 		if (g_unlink (filename) < 0)
 			g_assert_not_reached ();
+		g_free (filename);
 	}
 
 	g_dir_close (dir);
