@@ -950,6 +950,14 @@ rpc_C_OpenSession (CallState *cs)
 
 	/* Slot id becomes appartment so lower layers can tell clients apart. */
 
+	/*
+	 * IMPORTANT: When we open sessions on behalf of a client caller
+	 * we always specify CKF_G_APPLICATION_SESSION. This allows the module
+	 * to know whether they're talking to the daemon or a client app.
+	 *
+	 * This is a security feature.
+	 */
+
 	BEGIN_CALL (C_OpenSession);
 		IN_ULONG (slot_id);
 		IN_ULONG (flags);
@@ -2221,7 +2229,11 @@ gkm_rpc_layer_accept (void)
 
 	ds->socket = new_fd;
 
+#if GLIB_CHECK_VERSION(2,31,3)
+	ds->thread = g_thread_new ("dispatch", run_dispatch_thread, &(ds->socket));
+#else
 	ds->thread = g_thread_create (run_dispatch_thread, &(ds->socket), TRUE, &error);
+#endif
 	if (!ds->thread) {
 		gkm_rpc_warn ("couldn't start thread: %s", egg_error_message (error));
 		close (new_fd);
